import logging
from pytest import fixture
from stubilous.builder import Builder
logging.basicConfig(level=logging.DEBUG)


@fixture(scope="session")
def server(request):
    from multiprocessing import Process
    from stubilous.server import run
    builder = Builder()
    builder.server(host="localhost", port=9999)
    builder.route("GET", "/hello/<name>")("Hello {{ name }}!", 200)

    config = builder.build()
    proc = Process(target=run, args=(config,))

    def on_close():
        proc.terminate()
        proc.join()

    request.addfinalizer(on_close)
    proc.start()
    return proc


def test_builder_server():
    builder = Builder()
    builder.server("localhost", 9999)
    config = builder.build()

    assert config.host == "localhost"
    assert config.port == 9999


def test_builder_routes():
    builder = Builder()
    builder.route("GET", "/hello")("Hello world!")
    config = builder.build()
    route = config.routes[0]

    assert route.status == 200
    assert route.body() == "Hello world!"
    assert route.method == "GET"


def test_server_greeting(server):
    import requests
    result = requests.get("http://localhost:9999/hello/world")

    assert result.text == "Hello world!"



if __name__ == "__main__":
    from argparse import ArgumentParser
    import yaml

    from stubilous.server import run
    from stubilous.config import Config
    parser = ArgumentParser()
    parser.add_argument("--config", help="Service configuration file")
    args = parser.parse_args()
    with open(args.config, "r") as f:
        config = Config.from_dict(yaml.load(f))
    run(config)



from pytest import fixture

from stubilous.config import Config


@fixture
def config_file():
    return """
---
server:
  port: 80
  host: localhost
  routes:
      - desc: A test route
        method: GET
        path: /test
        body: Hello!
        status: 200
      - desc: Advanced route
        method: GET
        path: /test/<name>
        body: Hello {{name}}!
        status: 200
"""


@fixture
def basic_config(config_file):
    from six import StringIO
    import yaml
    buff = StringIO()
    buff.write(config_file)
    buff.seek(0)
    return Config.from_dict(yaml.load(buff))


def test_service_config(basic_config):
    assert basic_config.port == 80
    assert basic_config.host == "localhost"


def test_route_config(basic_config):
    route = basic_config.routes[0]
    assert route.method == "GET"
    assert route.path == "/test"
    assert route.body() == "Hello!"


def test_template_route(basic_config):
    route = basic_config.routes[1]
    assert route.method == "GET"
    assert route.body(name="test") == "Hello test!"



class Route(object):

    @staticmethod
    def from_dict(data):
        return Route(desc=data.get("desc"),
                     method=data["method"],
                     path=data["path"],
                     status=data["status"],
                     body=data.get("body"),
                     file=data.get("file"),
                     headers=data.get("headers"),
                     cookies=data.get("cookies")
                     )

    def __init__(self,
                 desc,
                 method,
                 path,
                 status=200,
                 body=None,
                 file=None,
                 headers=None,
                 cookies=None):
        self.desc = desc
        self.method = method
        self.path = path
        self.status = status
        if file:
            with open(self.file, "r") as f:
                self._body = f.read()
        else:
            self._body = body

        self.headers = headers
        self.cookies = cookies

    def body(self, **kwargs):
        from jinja2 import Template
        tmpl = Template(self._body)
        return tmpl.render(kwargs)


class Config(object):

    @staticmethod
    def from_dict(data):
        server = data["server"]
        routes = [Route.from_dict(r)
                  for r in server["routes"]]

        return Config(host=server["host"],
                      port=server["port"],
                      routes=routes)

    def __init__(self, host, port, routes):
        self.host = host
        self.port = port
        self.routes = routes



from functools import partial
from stubilous.config import Route


def response(callback,
             method,
             url,
             body,
             status=200,
             headers=None,
             cookies=None):

    callback(Route(method=method,
                   path=url,
                   body=body,
                   status=status,
                   desc="",
                   headers=headers,
                   cookies=cookies))


class Builder(object):

    def __init__(self):
        self.host = None
        self.port = None
        self.routes = []

    def server(self, host, port):
        self.host = host
        self.port = port
        return self

    def route(self, method, url):
        def callback(route):
            self.routes.append(route)
            return self

        return partial(response, callback, method, url)

    def build(self):
        from stubilous.config import Config
        return Config(host=self.host,
                      port=self.port,
                      routes=self.routes)



from flask import Flask, make_response


def create_app():
    return Flask(__name__)


def default_view(body, status=200, headers=None, cookies=None, **kwargs):
    response = make_response(body(**kwargs), status)
    if headers:
        response.headers = headers

    if cookies:
        response.cookies = cookies
    return response


def wrap(fn):
    import uuid
    fn.__name__ = str(uuid.uuid4())
    return fn


def init_routes(app, routes):
    from functools import partial
    for route in routes:
        app.add_url_rule(route.path,
                         methods=[route.method],
                         view_func=wrap(partial(default_view,
                                                route.body,
                                                route.status,
                                                route.headers)))


def run(config):
    app = create_app()
    init_routes(app, config.routes)
    app.run(host=config.host, port=config.port)



# -*- coding: utf-8 -*-
from setuptools import setup
from pip.req import parse_requirements

install_reqs = list(parse_requirements("requirements.txt", session={}))


def version():
    from stubilous import version
    return version.get_version()


setup(name="stubilous",
      version=version(),
      description="A plain simple Python http stub server",
      author="Šarūnas Navickas",
      author_email="zaibacu@gmail.com",
      url="https://github.com/CodersOfTheNight/stubilous",
      license="MIT",
      packages=["stubilous"],
      setup_requires=[str(ir.req) for ir in install_reqs] + ["pytest-runner"],
      test_suite="pytest",
      tests_require=["pytest", "requests"])
